//////////////////////////////////////////////////////////////////////////////////////////////////
//																								//
//											ATEV CNCL											//
//																								//
//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	Arguments (optional)
//		(int)argv[0]: atev id to start from;
//
//	Description
//		Search and cancel atevs of inactive
//		or deleted objects;
//
//	Desclaimer
//		Use at your own risk;
//
//	Affected objects
//		- atev w/o obj_id;
//		- object described in atev is not exist;
//		- ojbect described in atev is inactive;
//
//	Affect factories
//		iss, cr, chg, cnt, wf;
//
//	Options
//		- REP_STEP - show interval log;
//		- ILIMIT - iteration limit;
//		- MSGONCNCL - show message on atev cncl;
//
//	Backup command
//		pdm_extract -f "select * from Attached_Events where status_flag IN (2, 12)" > atev.txt
//
//	Restore command
//		pdm_extract -f "select * from Attached_Events where status_flag IN (2, 12)" > atev.txt
//
//	Run command
//		bop_cmd -f atev_cncl.frg "init()"
//
//	# cdtj71@gmail.com
//	# 2015-11-01
//
//////////////////////////////////////////////////////////////////////////////////////////////////

#define REP_STEP 300
#define ILIMIT 10000000
#define MSGONCNCL 1

void init(...) {
    printf("========================================\n");
    printf("Started\n");
    printf("========================================\n");

	int start_id, i, j, zcount;
	object dob, zfound, zobj, child;
	string fac;

	start_id = 0;
	if (argc > 0) {
		start_id = argv[0];
	}

	set_ilimit(ILIMIT);

	send_wait(0, top_object(), "call_attr", "atev", "sync_fetch", "MLIST_STATIC", format("status_flag IN (2, 12) AND id > %d", start_id), -1, 0);
	zcount = msg[1];
	if (zcount == 0) {
		printf("here we done! %d\n", start_id);
		logf(SIGNIFICANT, "here we done! %d\n", start_id);
		return;
	}
	zfound = msg[0];
	for (j=0;j<zcount;j++) {
		send_wait(0, zfound, "dob_by_index", "DEFAULT", j, j);
		zobj = msg[0];
		fac = getfac(zobj.obj_id);
		if (is_empty(fac)) {
			if (cnclme(zobj)) {
				logf(SIGNIFICANT, "[%s] canceled", zobj.persistent_id);
			}
		} else {
			send_wait(0, top_object(), "call_attr", fac, "dob_by_persid", NULL, zobj.obj_id);
			if (msg_error()) {
				if (cnclme(zobj)) {
					if (MSGONCNCL) logf(SIGNIFICANT, "[%s] canceled", zobj.persistent_id);
				}
			} else {
				child = msg[0];
				if (fac == "cr") {
					if (child.active == 0) {
						if (cnclme(zobj)) {
							if (MSGONCNCL) logf(SIGNIFICANT, "[%s] canceled", zobj.persistent_id);
						}
					}
				} else if (fac == "iss") {
					if (child.active == 0) {
						if (cnclme(zobj)) {
							if (MSGONCNCL) logf(SIGNIFICANT, "[%s] canceled", zobj.persistent_id);
						}
					}
				} else if (fac == "chg") {
					if (child.active == 0) {
						if (cnclme(zobj)) {
							if (MSGONCNCL) logf(SIGNIFICANT, "[%s] canceled", zobj.persistent_id);
						}
					}
				} else if (fac == "cnt") {
					if (child.delete_flag == 1) {
						if (cnclme(zobj)) {
							if (MSGONCNCL) logf(SIGNIFICANT, "[%s] canceled", zobj.persistent_id);
						}
					}
				} else if (fac == "wf") {
					if (child.chg.active == 0) {
						if (cnclme(zobj)) {
							if (MSGONCNCL) logf(SIGNIFICANT, "[%s] canceled", zobj.persistent_id);
						}
					}
				}
			}
		}
		if (j % REP_STEP == 0) {
			start_id = zobj.id;
			printf("%d of %d [Latest atev: %s]\n", j, zcount, zobj.persistent_id);
			logf(SIGNIFICANT, "%d of %d [Latest atev: %s]", j, zcount, zobj.persistent_id);
		}
	}

    printf("\n");
    printf("========================================\n");
    printf("Completed\n");
    printf("========================================\n");
}

int cnclme(object atev) {
	object gl;

	send_wait(0, top_object(), "get_co_group");
	if (msg_error()) {
		printf("Error : %s", msg[0]);
		return 0;
	}
	gl = msg[0];

	send_wait(0, gl, "checkout", atev);
	if (msg_error()) {
		printf("Error : %s", msg[0]);
		return 0;
	}
	
	send_wait(0, atev, "cancel_me");
	if (msg_error()) {
		printf("Error : %s", msg[0]);
		return 0;
	}
	
	send_wait(0, gl, "checkin");
	if (msg_error()) {
		printf("Error : %s", msg[0]);
		send_wait(0, gl, "uncheck");
		return 0;
	} else {
		// logf(SIGNIFICANT, "[%s] canceled", atev.persistent_id);
		return 1;
	}
}

string getfac(string obj_id) {
	string fac;
	// printf("%s\n", obj_id);
	fac = substr(obj_id, 0, sindex(obj_id, ":"));
	// printf("%s\n", fac);
	return fac;
}